#!/bin/sh

. /lib/functions.sh

set_rules() {
  if [ $(uci -q get rpcd.${1}.group) = "$2" ]; then
    $(uci -q delete rpcd.${1}.read)
    $(uci -q delete rpcd.${1}.write)

    for rule in $3; do
      rule=$(echo "$rule" | tr + \*)
      $(uci -q add_list rpcd.${1}.read=${rule})
    done
    for rule in $4; do
      rule=$(echo "$rule" | tr + \*)
      $(uci -q add_list rpcd.${1}.write=${rule})
    done
  fi
}

group="$1"

if [ "$group" = "admin" -o "$group" = "user" ]; then
  read_rules=$(uci -q get rpcd.${group}.read | tr \* +)
  read_access=$(uci -q get rpcd.${group}.target_read)
  write_rules=$(uci -q get rpcd.${group}.write | tr \* +)
  write_access=$(uci -q get rpcd.${group}.target_write)

  parsed_read_rules=""
  parsed_write_rules=""

  for rule in $read_rules; do
    if [ "$read_access" = "deny" ]; then
      parsed_read_rules="${parsed_read_rules} !${rule}*"
    elif [ "$read_access" = "allow" ]; then
      parsed_read_rules="${parsed_read_rules} ${rule}*"
    fi
  done

  if [ "$read_access" = "deny" ]; then
    parsed_read_rules="${parsed_read_rules} !superuser +"
  elif [ "$read_access" = "allow" ]; then
    parsed_read_rules="${parsed_read_rules} !superuser core"
  fi

  for rule in $write_rules; do
    if [ "$write_access" = "deny" ]; then
      parsed_write_rules="${parsed_write_rules} !${rule}*"
    elif [ "$write_access" = "allow" ]; then
      parsed_write_rules="${parsed_write_rules} ${rule}+"
    fi
  done

  if [ "$write_access" = "deny" ]; then
    parsed_write_rules="${parsed_write_rules} !superuser +"
  elif [ "$write_access" = "allow" ]; then
    parsed_write_rules="${parsed_write_rules} !superuser core"
  fi

  config_load rpcd
  config_foreach set_rules login "$group" "${parsed_read_rules}" "${parsed_write_rules}"
  uci commit rpcd
fi
