import sys
import utility_integration as util
from utils.ssh import open_ssh_connection, send_cmd
from datetime import datetime
import json
import pytz
sys.path.append("../../../../tests")

class test_firmware(util.WrapTest):
    
    def test_firmware_base_functionality(self):
        base_url = "/system/firmware"
        kernel = ""
        version = ""
        build_date = ""
        modems = []
        with self.subTest("get_info"):
            with open_ssh_connection() as ssh:
                kernel = send_cmd(ssh, "uname -r").strip()
                version = send_cmd(ssh, "cat /etc/version").strip()
                build_date = int(send_cmd(ssh, "cat /etc/firmware-date").strip())
                build_date = str(datetime.fromtimestamp(build_date).astimezone(pytz.UTC).strftime("%Y-%m-%d %H:%M:%S"))
                available_modems = send_cmd(ssh, "ubus list gsm.modem*")
                for available_modem in available_modems.splitlines():
                    info = json.loads(send_cmd(ssh, "ubus call " + available_modem + " info"))
                    single_modem = {}
                    single_modem["type"] = "Internal modem" if info["builtin"] == True else "External modem"
                    single_modem["version"] = info["cache"]["firmware"]
                    single_modem["id"] = info["usb_id"]
                    modems.append(single_modem)
        with self.subTest("get_fw_info"):
            x = self.get(base_url + "/status/device")
            x.assert_data({
                "kernel_version":kernel,
                "version":version,
                "build_date":build_date
            })
        with self.subTest("get_modems_info"):
            x = self.get(base_url + "/status/modem")
            resp = x.resp.json()
            self.assertEqual(modems, resp["data"]["modems"])

    @util.allow_device("RUTX")
    def test_firmware_upload_fw(self):
        base_url = "/system/firmware"
        with self.subTest("upload_fw"):
            x = self.send_file(base_url + "/device", "files/RUTX_FW.bin")
            x.assert_data({
                "valid": "1",
                "passwd_warning": "0",
                "authorized": "1",
                "message_code": "0",
                "size": "22.44 MB",
                "newer": "0",
                "bootloader_valid": "1",
                "sha256": "193ae47a5849ad7c05b658bf32c929bde21bd3f3395bd36eb243dea3e54ce83c",
                "md5": "c8b63e143a28a2b633039c007fc379c5",
            })
        with self.subTest("verify_fw"):
            x = self.post(base_url + "/actions/verify", None)
            x.assert_data({
                "valid": "1",
                "sha256": "193ae47a5849ad7c05b658bf32c929bde21bd3f3395bd36eb243dea3e54ce83c",
                "passwd_warning": "0",
                "authorized": "1",
                "md5": "c8b63e143a28a2b633039c007fc379c5",
                "message_code": "0",
                "size": "22.44 MB",
                "newer": "0",
                "bootloader_valid": "1"
            })