from response_codes import ResponseCodes
import utility_integration as util
import sys
import random
from utils.general_api import get_modems

from utils.ssh import open_ssh_connection
sys.path.append("../../../../tests")
BASE_URL = "/services/ntp/client/config"
OPTIONS_URL = "/services/ntp/client/options"
ALWAYS_AVAILABLE_OPTIONS = {"enabled": "",
                            "zoneName": "",
                            "timezone": "",
                            "freq": "",
                            "current_system_time": "",
                            "force": "",
                            "save": "",
                            "count": "",
                            "interval": ""
                            }


class test_ntp_client(util.WrapTest):
    def setUp(self) -> None:
        x = self.get(BASE_URL + "/general")
        x.assert_code(200)
        self.original_section = x.resp.json()["data"]

    def tearDown(self) -> None:
        cfg = {}
        for opt in ALWAYS_AVAILABLE_OPTIONS:
            cfg[opt] = self.original_section.get(opt) or ""
        for opt in self.original_section:
            cfg[opt] = self.original_section.get(opt) or ""
        del cfg["timezone"]  # readonly option
        del cfg["current_system_time"]

        x = self.put_data(BASE_URL + "/general", cfg)
        x.assert_code(200)

    def test_ntp_time_zone_validation(self):
        x = self.get(OPTIONS_URL)
        x.assert_code(200)
        timezones = x.resp.json()["data"]["timezones"]

        # checking only 10 random timezones, because checking all timezones takes over a minute
        selected_timezones = random.choices(timezones, k=10)
        for tz in selected_timezones:
            x = self.put_data(BASE_URL + "/general", {"zoneName": tz})
            x.assert_code(200)
            s = x.resp.json()["data"]
            self.assertEqual(s["zoneName"], tz)

        x = self.put_data(BASE_URL + "/general", {"zoneName": "invalid_zonename"})
        x.assert_code(422)

    def test_ntp_time_zone_custom_options(self):
        # "zoneName" option
        x = self.put_data(BASE_URL + "/general", {"zoneName": "America/Asuncion"})
        x.assert_code(200)
        self.assertEqual(x.json["data"]["zoneName"], "America/Asuncion")
        self.assertEqual(x.json["data"]["timezone"], "<-04>4<-03>,M10.1.0/0,M3.4.0/0")
        with open_ssh_connection() as ssh:
            res = ssh.send_cmd("cat /etc/TZ")
            self.assertEqual(res.strip(), "<-04>4<-03>,M10.1.0/0,M3.4.0/0")
            res = ssh.send_cmd("grep \"option timezone '<-04>4<-03>,M10.1.0/0,M3.4.0/0'\" /etc/config/system &> /dev/null ; echo $?")
            self.assertEqual(res.strip(), "0")
            res = ssh.send_cmd("grep \"option zoneName 'America/Asuncion'\" /etc/config/system &> /dev/null ; echo $?")
            self.assertEqual(res.strip(), "0")

        # "freq" option
        x = self.put_data(BASE_URL + "/general", {"freq": "123"})
        x.assert_code(200)
        self.assertEqual(x.json["data"]["freq"], "123")
        self.assertEqual(self.get_section("ntpclient", "@ntpdrift[0]")["values"]["freq"], "123")

        # gps options
        x = self.get("/system/device/info")
        x.assert_code(200)
        hwinfo = x.json["data"]["board"]["hwinfo"]
        if hwinfo.get("gps"):
            gps_enabled = None
            with self.subTest("get gps cfg"):
                x = self.get("/services/gps/general/config/general")
                gps_enabled = x.json["data"].get("enabled")

            with self.subTest("test gps options"):
                x = self.put_data(BASE_URL + "/general", {"gps_sync": "1", "gps_interval": "300", })
                x.assert_code(200)
                self.assertEqual(x.json["data"]["gps_sync"], "1")
                self.assertEqual(x.json["data"]["gps_interval"], "300")
                gps_s = self.get_section("gps", "gpsd")["values"]
                self.assertEqual(gps_s["enabled"], "1")
                if gps_enabled != "1":
                    self.assertDictEqual(x.json["messages"][0], {"code": 2, "message": "GPS has been enabled", "source": "gps_sync"})

                x = self.put_data(BASE_URL + "/general", {"gps_sync": "0", "gps_interval": "", })
                x.assert_code(200)
                self.assertEqual(x.json["data"]["gps_sync"], "0")

            with self.subTest("reset gps cfg"):
                x = self.put_data("/services/gps/general/config/general", {"enabled": gps_enabled or ""})
                x.assert_code(200)
        else:
            x = self.put_data(BASE_URL + "/general", {"gps_sync": "1", "gps_interval": "300", })
            x.assert_code(422)


        # mobile options
        modems = get_modems(self)
        if len(modems) > 0:
            x = self.put_data(BASE_URL + "/general", {"sync_enabled": "1", "tmz_sync_enabled": "1", "failover": "10"})
            x.assert_code(200)
            x = self.put_data(BASE_URL + "/general", {"sync_enabled": "", "tmz_sync_enabled": "", "failover": ""})
            x.assert_code(200)
        else:
            x = self.put_data(BASE_URL + "/general", {"sync_enabled": "1", "tmz_sync_enabled": "1", "failover": "10"})
            x.assert_code(422)


    def test_ntp_client_deletion(self):
        x = self.delete(BASE_URL + "/general")
        x.assert_error("Validation", "Section deletion is not allowed", ResponseCodes.NO_DELETE.val())

    def test_ntp_client_creation(self):
        x = self.post_data(BASE_URL, {})
        x.assert_error("Validation", "Section creation is not allowed", ResponseCodes.NO_CREATE.val())
