import sys
sys.path.append("../../../../tests")
from utility_integration import WrapTest, Env
from utils.ssh import open_ssh_connection, is_process_running

class TR069(WrapTest):
    url = "/services/tr069/config/general"
    firewall_url = "/network/firewall/traffic_rules/config"

    def test_basic_update(self):
        x = self.put_data(self.url, {
            "enabled": "0",
            ".type": "acs",
            "password": "easycwmp",
            "periodic_interval": "100",
            "username": "easycwmp",
            "periodic_enable": "1",
            "server_request": "0",
            "url": "http://192.168.1.110:8080/openacs/acs"
        })
        x.assert_data({
            "enabled": "0",
            ".type": "acs",
            "id": "general",
            "password": "easycwmp",
            "periodic_interval": "100",
            "username": "easycwmp",
            "periodic_enable": "1",
            "server_request": "0",
            "url": "http://192.168.1.110:8080/openacs/acs"
        })

    def test_check_firewall_rule(self):
        x = self.get(self.url) # used to skip the test if pkg not installed
        x.assert_code(200)

        with self.subTest("Enable Firewall rule"):
            x = self.put_data(self.url, {
                "server_request": "1",
            })
            x.assert_code(200)

        with self.subTest("Check firewall rule"):
            x = self.get(self.firewall_url)
            resp = x.resp
            found = False
            for section in resp.json()['data']:
                if section['name'] == "Allow_TR069_server_request":
                    found = True        
                    self.assertEqual(section['target'], "ACCEPT")
                    self.assertEqual(section['proto'], "tcp")
                    self.assertListEqual(section['dest_port'], ["7547"])
                    self.assertEqual(section['enabled'], "1")

            if not found:
                self.fail("Firewall rule is not created")

    def test_check_process(self):
        with open_ssh_connection() as ssh:
            self.assertFalse(is_process_running(ssh, "easycwmp"), "Expected 'easycwmp' not to be running")

            x = self.put_data(self.url, {
                 "enabled": "1"
            })
            x.assert_code(200)

            self.assertTrue(is_process_running(ssh, "easycwmp"), "Expected 'easycwmp' to be running")

            x = self.put_data(self.url, {
                "enabled": "0"
            })
            x.assert_code(200)