import sys
sys.path.append("../../../../tests")
from utility_integration import WrapTest, Env
from utils.ssh import open_ssh_connection, is_process_running, is_process_stopped, send_cmd
from utils.general_api import is_package_installed

class Ullog(WrapTest):
    url_interfaces = "/network/interfaces/config"
    url_ulog_settings = "/services/ulog/general/config/general"
    url_ulog_options = "/services/ulog/general/options"
    url_ulog_ftp = "/services/ulog/ftp/config"
    url_ulog_status = "/services/ulog/general/status"

    ulog_ftp_config = {
        "host": "1.1.1.1",
        "username": "username",
        "password": "pswd",
        "port": "420",
        "extra_name_info": "none",
        "custom_string": "customString",
        "fixed": "0",
        "hours": "4",
        "minutes": "20",
        "interval": "1",
        "weekdays": ["mon", "tue", "wed", "thu", "fri", "sat", "sun"],
        ".type": "server"
    }

    ulog_ftp_empty_config = {
        "host": "",
        "username": "",
        "password": "",
        "port": "",
        "extra_name_info": "",
        "custom_string": "",
        "fixed": "",
        "hours": "",
        "minutes": "",
        "interval": "",
        "weekdays": ""
    }

    def test_networks_options(self):
        with open_ssh_connection() as ssh:
            with self.subTest("create another lan section"):
                x = self.get(self.url_interfaces + "/lan")
                if x.code == 200:
                    lan_s = x.json["data"]
                    lan_s["id"] = "test_lan2"
                    x = self.post_data(self.url_interfaces, lan_s)
                    x.assert_code(201)

            with self.subTest("main test"):
                options = []

                x = self.get(self.url_interfaces)
                x.assert_code(200)

                for interface in x.resp.json()["data"]:
                    if "ifname" in interface and ("eth0" in interface["ifname"] or "rndis0" in interface["ifname"] or
                    "ecm0" in interface["ifname"] or "br-lan" in interface["ifname"] or "wlan" in interface["ifname"] or "eth0.1" in interface["ifname"]):
                        options.append(interface["id"])

                wireless = send_cmd(ssh, f"ls /etc/config/wireless &> /dev/null ; echo $?")
                chilli = send_cmd(ssh, f"ls /etc/config/chilli &> /dev/null ; echo $?")
                if wireless.strip() ==  "0" and chilli.strip() == "0":            
                    options.append("hotspot")

                x = self.get(self.url_ulog_options)
                x.assert_code(200)
                
                response = x.resp.json()["data"]["network"]
                response.sort()
                options.sort()

                self.assertListEqual(response, options)

            with self.subTest("delete created lan section"):
                self.delete(self.url_interfaces + "/test_lan2").assert_code(200)

    def test_ulog_ftp(self):
        x = self.get(self.url_ulog_ftp)
        x.assert_code(200)
        sid = x.resp.json()["data"][0]["id"]

        x = self.put_data(f"{self.url_ulog_ftp}/{sid}", self.ulog_ftp_config)
        self.ulog_ftp_config.update({"id": sid})
        x.assert_data(self.ulog_ftp_config)

        x = self.put_data(f"{self.url_ulog_ftp}/{sid}", self.ulog_ftp_empty_config)
        x.assert_code(200)

    def test_ulog_file(self):
        file_path = self.get_section("ulogd", "emu1")["values"]["file"]
        log_file_text = "TestTestTestTest"
        with open_ssh_connection() as ssh:
            send_cmd(ssh, "rm " + file_path)
            send_cmd(ssh, f'echo \'{log_file_text}\' > {file_path}')
        
        x = self.get(self.url_ulog_status)
        x.assert_code(200)
        traffic_log = x.resp.json()["data"]["traffic_log"]
        
        self.assertEqual(traffic_log, (log_file_text+"\n"))

    def test_check_process(self):
        x = self.put_data(self.url_ulog_settings, {
            "enabled": "0"
        })
        x.assert_code(200)

        with open_ssh_connection() as ssh:
            self.assertTrue(is_process_stopped(ssh, "ulogd"), "Expected 'ulogd' not to be running")

            x = self.put_data(self.url_ulog_settings, {
                "enabled": "1"
            })
            x.assert_code(200)

            self.assertTrue(is_process_running(ssh, "ulogd"), "Expected 'ulogd' to be running")