#!/bin/sh
. /lib/netifd/netifd-wireless.sh

init_wireless_driver "$@"
maclist=""
force_channel=""

drv_ralink_init_device_config() {
	config_add_string country region variant log_level short_preamble
	config_add_boolean noscan
	config_add_int channel frag rts txpower beacon_int
}

drv_ralink_init_iface_config() {
	config_add_string 'auth_server:host'
	config_add_string auth_secret
	config_add_int 'auth_port:port'

	config_add_string ifname apname mode wifi_id bssid ssid encryption key key1 key2 key3 key4 wps_pushbutton macfilter led wmm
	config_add_int dtim_period wpa_group_rekey max_inactivity max_listen_interval
        config_add_boolean hidden isolate disassoc_low_ack skip_inactivity_poll txburst
        config_add_array 'maclist:list(macaddr)'
}

drv_ralink_cleanup() {
	logger drv_ralink_cleanup
}

ralink_setup_ap(){
	local name="$1"
	local eap=0
	local rssi_threshold=-92

	json_select data
	json_get_vars ifname
	json_select ..

	json_select config
	json_get_vars mode ssid encryption key key1 key2 \
		key3 key4 wps_pushbutton server port hidden \
		isolate macfilter wmm dtim_period wpa_group_rekey \
		disassoc_low_ack skip_inactivity_poll \
		max_inactivity max_listen_interval txburst
	json_get_values maclist maclist

	ifconfig $ifname up

	set_default wmm 1
	set_default dtim_period 3
	set_default disassoc_low_ack 1
	set_default skip_inactivity_poll 0
	set_default txburst 0
	set_default max_inactivity 0

	[ -z "$force_channel" ] || iwpriv $ifname set Channel=$force_channel
	[ "$isolate" = 1 ] || isolate=0
	iwpriv $ifname set NoForwarding=$isolate
	iwpriv $ifname set NoForwardingBTNBSSID=$isolate
	iwpriv $ifname set NoForwardingMBCast=$isolate

	[ "$hidden" = 1 ] || hidden=0
	iwpriv $ifname set HideSSID=$hidden

	wsc_methods=0
	wsc_mode=0
	case "$encryption" in
	psk*|mixed*|sae*)
		local enc="WPA2PSKWPA3PSK"
		case "$encryption" in
			psk | psk+*) enc=WPAPSK;;
			psk2 | psk2*) enc=WPA2PSK;;
			mixed*) enc=WPAPSKWPA2PSK;;
			wpa | wpa+*) eap=1; enc=WPA;;
			wpa2*) eap=1; enc=WPA2;;
			psk3-mixed | sae-mixed) enc=WPA2PSKWPA3PSK;;
                        psk3 | sae) enc=WPA3PSK;;
		esac
		local crypto="AES"
		case "$encryption" in
			psk3* | sae*) ;;
			*tkip+aes*) crypto=TKIPAES;;
			*tkip*) crypto=TKIP;;
		esac
		[ "$eap" = "1" ] && {
			iwpriv $ifname set RADIUS_Server=$server
			iwpriv $ifname set RADIUS_Port=$port
			iwpriv $ifname set RADIUS_Key=$key
			iwpriv $ifname set own_ip_addr=192.168.1.1
			iwpriv $ifname set IEEE8021X=0
			iwpriv $ifname set EAPifname=eth0.2
			iwpriv $ifname set PreAuthIfname=br-lan
		}
		iwpriv $ifname set AuthMode=$enc
		iwpriv $ifname set EncrypType=$crypto
		iwpriv $ifname set IEEE8021X=0
		iwpriv $ifname set "SSID=${ssid}"
		[ "$eap" = "1" ] || iwpriv $ifname set "WPAPSK=${key}"
		iwpriv $ifname set DefaultKeyID=2
		iwpriv $ifname set "SSID=${ssid}"
		if [ "$wps_pushbutton" == "1" ]; then
			wsc_methods=128
			wsc_mode=7
		fi
		;;
	wep*)
		local enc="WEPAUTO"
		iwpriv $ifname set EncrypType=WEP
		iwpriv $ifname set IEEE8021X=0

		case "$encryption" in
                        *open) enc=OPEN;;
                        *shared) enc=SHARED;;
                esac
		for idx in 1 2 3 4; do
			json_get_var keyn key${idx}
			[ -n "$keyn" ] && iwpriv $ifname set "Key${idx}=${keyn}"
		done
		iwpriv $ifname set AuthMode=$enc
		iwpriv $ifname set DefaultKeyID=${key}
		iwpriv $ifname set "SSID=${ssid}"
		;;
	none)
		iwpriv $ifname set "SSID=${ssid}"
		;;
	owe)
		iwpriv $ifname set AuthMode=OWE
		iwpriv $ifname set EncrypType=AES
		iwpriv $ifname set "SSID=${ssid}"
		iwpriv $ifname set IEEE8021X=0
		;;
	esac

	iwpriv $ifname set WscConfMode=$wsc_mode
	iwpriv $ifname set WscConfStatus=2
	iwpriv $ifname set WscMode=2
	iwpriv $ifname set ACLClearAll=1
	[ -n "$maclist" ] && {
		for m in ${maclist}; do
			logger iwpriv $ifname set ACLAddEntry="$m"
			iwpriv $ifname set ACLAddEntry="$m"
		done
	}
	case "$macfilter" in
	allow)
		iwpriv $ifname set AccessPolicy=1
		;;
	deny)
		iwpriv $ifname set AccessPolicy=2
		;;
	*)
		iwpriv $ifname set AccessPolicy=0
		;;
	esac

	iwpriv $ifname set WmmCapable="${wmm}"
	iwpriv $ifname set DtimPeriod="${dtim_period}"

	[ -n "$wpa_group_rekey" ] && iwpriv $ifname set RekeyInterval="${wpa_group_rekey}" 

	iwpriv $ifname set disassoc_low_ack="${disassoc_low_ack}"
	iwpriv $ifname set skip_inactivity_poll="${skip_inactivity_poll}"
	iwpriv $ifname set TxBurst="${txburst}"

	[ "$max_inactivity" -gt 0 ] && iwpriv $ifname set IdleTimeout="${max_inactivity}"
	[ "$max_listen_interval" -gt 0 ] && iwpriv $ifname set max_listen_interval="${max_listen_interval}"
	iwpriv $ifname set KickStaRssiLow="${rssi_threshold}"
	json_select ..


	wireless_add_vif "$name" "$ifname"
}

ralink_setup_sta(){
	local name="$1"
	local bcn_active=0
	json_select config
	json_get_vars mode apname ifname ssid bssid encryption key key1 key2 key3 key4 wps_pushbutton led

	linkit_mode="$(uci get wireless.radio0.linkit_mode)"
	[ "${linkit_mode}" = "ap" ] && return
	[ "${linkit_mode}" == "apsta" ] && bcn_active=1

	key=
	case "$encryption" in
		psk*|mixed*) json_get_vars key;;
		wep) json_get_var key key1;;
	esac
	json_select ..

	/sbin/ap_client "ra0" "$ifname" "${ssid}" "${key}" "${bssid}" "${bcn_active}" "${led}"
	sleep 1
	wireless_add_process "$(cat /tmp/apcli-${ifname}.pid)" /sbin/ap_client ra0 "$ifname" "${ssid}" "${key}" "${bssid}" "${bcn_active}" "${led}"

	wireless_add_vif "$name" "$ifname"
}

ralink_count_ap() {
	json_select config

	json_get_vars ifname

	[ -n "$ifname" ] || ifname="ra${if_idx}"

	if_idx=$((if_idx + 1))

	json_select ..

	json_add_object data
	json_add_string ifname "$ifname"
	json_close_object
}

drv_ralink_setup() {
	local module="mt7628"
	local bcn_active=0
	wmode=9
	VHT=0
	VHT_SGI=0
	HT=0
	EXTCHA=0
	if_idx=0
	local ref


	json_select config
	json_get_vars variant region country channel htmode log_level \
		short_preamble noscan:0 rts:2347 frag:2346 txpower beacon_int:100
	json_select ..

	[ -z "$region" ] && region=0

	if [ -z "$txpower" ] || [ "$txpower" -gt "20" ]; then
		txpower=100
	else
		txpower="$(echo $txpower | awk '{ print int(10^($1/10)) }')"
	fi

	[ "$beacon_int" -gt "1024" ] && beacon_int=1024
	[ "$beacon_int" -lt "20" ] && beacon_int=20

	[ "$short_preamble" = 1 ] || short_preamble=0

	case ${htmode:-none} in
	HT20)
		wmode=10
		HT=0
		;;
	HT40)
		wmode=10
		HT=1
		EXTCHA=1
		;;
	VHT20)
		wmode=15
		HT=0
		VHT=0
		VHT_SGI=1
		;;
	VHT40)
		wmode=15
		HT=1
		VHT=0
		VHT_SGI=1
		EXTCHA=1
		;;
	VHT80)
		wmode=15
		HT=1
		VHT=1
		VHT_SGI=1
		EXTCHA=1
		;;
	*)
		case $hwmode in
		a) wmode=2;;
		g) wmode=3;;
		esac
		;;
	esac

	if [ "$auto_channel" -gt 0 ]; then
		force_channel=""
		channel=0
		auto_channel=3
	else
		force_channel=$channel
		auto_channel=0
	fi

	coex=1
	[ "$noscan" -gt 0 ] && coex=0

	for_each_interface "ap" ralink_count_ap

	cat > /tmp/${module}.dat<<EOF
#The word of "Default" must not be removed
Default
BssidNum=${if_idx}
Beacon=${bcn_active}
HT_BW=${HT:-0}
HT_EXTCHA=${EXTCHA:-0}
CountryRegion=${region}
CountryCode=${country}
WirelessMode=${wmode:-9}
Channel=${channel:-8}
AutoChannelSelect=$auto_channel
Debug=${log_level:-3}
SSID=${ssid:-OpenWrt}
HT_BSSCoexistence=$coex
WscConfMethods=680
RTSThreshold=${rts}
FragThreshold=${frag}
TxPower=${txpower}
BeaconPeriod=${beacon_int}
EOF

	cat /etc/wireless/${module}_tpl.dat >> /tmp/${module}.dat

	if [ ! -e "/etc/wireless/${module}/${module}.dat" ]; then
		ln -s /tmp/${module}.dat /etc/wireless/${module}/${module}.dat
	fi

	# in some case we have to reload drivers. (mbssid)
	ref=$(cat /sys/module/$module/refcnt)

	if [ "$ref" = "0" ]; then
		rmmod $module
		insmod $module
	fi


	for_each_interface "ap" ralink_setup_ap
	for_each_interface "sta" ralink_setup_sta
	wireless_set_up
}

ralink_teardown() {
	json_select data
	json_get_vars ifname
	json_select ..

	ifconfig $ifname down

}

drv_ralink_teardown() {
	wireless_process_kill_all
	for_each_interface "ap sta" ralink_teardown
}

add_driver ralink
