#
# MT76x8 Profiles
#

include common-teltonika.mk

KERNEL_LOADADDR := 0x80000000

define Device/tlt-mt7628-common
	SOC := mt7628an

	DEVICE_FEATURES := small_flash
	MTDPARTS :=
	BLOCKSIZE := 64k
	KERNEL := kernel-bin | append-dtb | lzma | uImage lzma
	KERNEL_INITRAMFS := kernel-bin | append-dtb | lzma | uImage lzma
	DEVICE_DTS = $$(SOC)_$(1)

	UBOOT_SIZE := 131072
	CONFIG_SIZE := 65536
	ART_SIZE := 196608
	NO_ART := 0
	IMAGE_SIZE := 15424k
	MASTER_IMAGE_SIZE := 16384k

	IMAGE/sysupgrade.bin = \
			append-kernel | pad-to $$$$(BLOCKSIZE) | \
			append-rootfs | pad-rootfs | append-metadata | \
			check-size $$$$(IMAGE_SIZE) | finalize-tlt-webui

	IMAGE/master_fw.bin = \
			append-tlt-uboot | pad-to $$$$(UBOOT_SIZE) | \
			append-tlt-config | pad-to $$$$(CONFIG_SIZE) | \
			append-tlt-art | pad-to $$$$(ART_SIZE) | \
			append-kernel | pad-to $$$$(BLOCKSIZE) | \
			append-rootfs | pad-rootfs | \
			check-size $$$$(MASTER_IMAGE_SIZE) | \
			finalize-tlt-master-stendui
endef

define Device/teltonika_tap100
	$(Device/tlt-mt7628-common)
	DEVICE_MODEL := TAP100
	DEVICE_FEATURES := vendor_wifi
	DEVICE_INITIAL_FIRMWARE_SUPPORT := 7.4
	UBOOT_NAME := tlt-tap100
	# Default common packages for TAP100 series
	# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	# Wireless related:
	DEVICE_PACKAGES += kmod-mt7628
	# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
endef
TARGET_DEVICES += teltonika_tap100

define Device/teltonika_otd140
	$(Device/tlt-mt7628-common)
	IMAGE/master_fw.bin = \
		append-tlt-uboot | pad-to $$$$(UBOOT_SIZE) | \
		append-tlt-config | pad-to $$$$(CONFIG_SIZE) | \
		append-kernel | pad-to $$$$(BLOCKSIZE) | \
		append-rootfs | pad-rootfs | \
		check-size $$$$(MASTER_IMAGE_SIZE) | \
		finalize-tlt-master-stendui
	DEVICE_MODEL := OTD140
	UBOOT_NAME := tlt-otd140
	DEVICE_INITIAL_FIRMWARE_SUPPORT := 7.4.4
	# Default common packages for OTD140 series
	# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	# USB related:
	DEVICE_PACKAGES += kmod-usb2 kmod-usb-ohci kmod-usb-ledtrig-usbport kmod-i2c-mt7628
	# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
endef
TARGET_DEVICES += teltonika_otd140

define Device/teltonika_rut2m
	$(Device/tlt-mt7628-common)
	DEVICE_MODEL := RUT2M
	UBOOT_NAME := tlt-rut2m
	# Default common packages for RUT241, RUT200 series
	# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	# USB related:
	DEVICE_PACKAGES += kmod-usb2 kmod-usb-ohci kmod-usb-ledtrig-usbport

	# Wireless related:
	DEVICE_PACKAGES += kmod-mt76x0e kmod-mt7603
	# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
endef
TARGET_DEVICES += teltonika_rut2m

define Device/teltonika_rut301
	$(Device/tlt-mt7628-common)
	IMAGE/master_fw.bin = \
		append-tlt-uboot | pad-to $$$$(UBOOT_SIZE) | \
		append-tlt-config | pad-to $$$$(CONFIG_SIZE) | \
		append-kernel | pad-to $$$$(BLOCKSIZE) | \
		append-rootfs | pad-rootfs | \
		check-size $$$$(MASTER_IMAGE_SIZE) | \
		finalize-tlt-master-stendui
	DEVICE_MODEL := RUT301
	DEVICE_INITIAL_FIRMWARE_SUPPORT := 7.4.1
	UBOOT_NAME := tlt-rut301
	# Default common packages for RUT301
	# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	# USB related:
	DEVICE_PACKAGES += kmod-usb-core kmod-usb2 kmod-usb-ohci kmod-usb-serial kmod-usb-acm \
		kmod-usb-serial-ch341 kmod-usb-serial-pl2303 kmod-usb-serial-ark3116 \
		kmod-usb-serial-belkin kmod-usb-serial-cp210x kmod-usb-serial-cypress-m8 \
		kmod-usb-serial-ftdi
	# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
endef
TARGET_DEVICES += teltonika_rut301

define Device/teltonika_rut361
	$(Device/tlt-mt7628-common)
	DEVICE_MODEL := RUT361
	UBOOT_NAME := tlt-rut361
	DEVICE_INITIAL_FIRMWARE_SUPPORT := 7.4.1
	# Default common packages for RUT361
	# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	# USB related:
	DEVICE_PACKAGES += kmod-usb2 kmod-usb-ohci

	# Wireless related:
	DEVICE_PACKAGES += kmod-mt76x0e kmod-mt7603
	# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
endef
TARGET_DEVICES += teltonika_rut361

define Device/teltonika_rut9m
	$(Device/tlt-mt7628-common)
	DEVICE_MODEL := RUT9M
	UBOOT_NAME := tlt-rut9m
	# Default common packages for RUT9M series
	# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	# Essential must-have:
	DEVICE_PACKAGES := kmod-spi-gpio kmod-gpio-nxp-74hc164 kmod-i2c-mt7628 \
			kmod-hwmon-core kmod-hwmon-mcp3021 kmod-cypress-serial

	# USB related:
	DEVICE_PACKAGES += kmod-usb-core kmod-usb2 kmod-usb-serial kmod-usb-acm \
			kmod-usb-serial-ch341 kmod-usb-serial-pl2303 kmod-usb-serial-ark3116 \
			kmod-usb-serial-belkin kmod-usb-serial-cp210x kmod-usb-serial-cypress-m8 \
			kmod-usb-serial-ftdi

	# Wireless related:
	DEVICE_PACKAGES += kmod-mt76x0e kmod-mt7603
	# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	HW_MODS := mod1%2c7c_6005
endef
TARGET_DEVICES += teltonika_rut9m
